'use strict';

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssValueParser = require('postcss-value-parser');

var _postcssValueParser2 = _interopRequireDefault(_postcssValueParser);

var _humps = require('humps');

var _humps2 = _interopRequireDefault(_humps);

var _promisePolyfill = require('promise-polyfill');

var _promisePolyfill2 = _interopRequireDefault(_promisePolyfill);

var _resetCore = require('./reset-core');

var _resetCore2 = _interopRequireDefault(_resetCore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

if (typeof window !== 'undefined') {
  window.Promise = window.Promise || _promisePolyfill2.default;
} else if (typeof global !== 'undefined') {
  global.Promise = global.Promise || _promisePolyfill2.default;
}

var atRules = {
  global(platefprm) {
    return _resetCore2.default.resetGlobal(platefprm);
  },
  nested() {
    for (var _len = arguments.length, tags = Array(_len), _key = 0; _key < _len; _key++) {
      tags[_key] = arguments[_key];
    }

    return _resetCore2.default.resetNested(tags);
  }
};

var ampInsertedNodes = {};

var unwrapAmp = function unwrapAmp(nodeSelector, node) {
  if (nodeSelector.indexOf('&:') >= 0 && node.name !== 'media') {
    return node.selectors.map(function (selector) {
      return nodeSelector.replace(/&/g, selector);
    }).join(',');
  }
  return nodeSelector;
};

var getGlobalSelector = function getGlobalSelector(node) {
  if (node.parent && node.parent.type === 'atrule') {
    return `${node.parent.name} ${node.parent.params} ${node.selector}`;
  } else if (node.name === 'media') {
    return getGlobalSelector(node.parent);
  }
  return node.selector;
};

var applyRuleSetToNode = function applyRuleSetToNode(ruleSet, node, currentAtRule) {
  Object.keys(ruleSet).forEach(function (prop) {
    var rule = ruleSet[prop];
    if (typeof rule === 'object') {
      if (node.name !== 'media') {
        var extRule = _postcss2.default.rule({ selector: unwrapAmp(prop, node) });
        applyRuleSetToNode(rule, extRule);

        var globalSelector = getGlobalSelector(node);
        node.parent.insertAfter(ampInsertedNodes[globalSelector] || node, extRule);
        ampInsertedNodes[globalSelector] = extRule;
      } else {
        var mediaNestedRule = _postcss2.default.parse(`${prop} ${JSON.stringify(rule).replace(/"/g, '')}`);
        node.append(mediaNestedRule);
      }
    } else {
      if (currentAtRule) {
        node.insertBefore(currentAtRule, { prop: prop, value: rule });
      } else {
        node.append({ prop, value: rule });
      }
    }
  });
};

var stringifyNode = function stringifyNode(node) {
  if (node.type === 'function') {
    return (node.before || '') + node.value + '(' + node.nodes.map(stringifyNode).join('') + ')' + (node.after || '');
  } else {
    return (node.before || '') + node.value + (node.after || '');
  }
};

var isPromise = function isPromise(obj) {
  return !!obj && (typeof obj === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
};

module.exports = _postcss2.default.plugin('postcss-simple-reset', function () {
  return function (root) {
    var promises = [];
    root.walkAtRules('reset', function (rule) {
      var ruleName = rule.params.split(/\(/, 1)[0].trim();
      var ruleFun = atRules[_humps2.default.camelize(ruleName)];
      if (ruleFun) {
        var params = [];
        var parsed = (0, _postcssValueParser2.default)(rule.params);
        parsed.nodes.forEach(function (node) {
          if (node.type === 'function') {
            node.nodes.forEach(function (i) {
              if (i.type === 'word' || i.type === 'function' || i.type === 'string') {
                params.push(stringifyNode(i));
              }
            });
          }
        });

        var promise = ruleFun.apply(undefined, params);

        if (isPromise(promise)) {
          promises.push(promise);
        }

        promise.then(function (resolve) {
          if (typeof resolve === 'object') {
            applyRuleSetToNode(resolve, rule.parent, rule);
          } else {
            root.prepend(resolve);
          }
          rule.remove();
        }).catch(console.error);
      } else {
        throw rule.error('Unknown utility ' + ruleName);
      }
    });
    return _promisePolyfill2.default.all(promises);
  };
});